#include "Functions.h"
#include <math.h>



static gsl_rng *r;  /*static reference to the generator*/
static unsigned long int sd=0; /*static reference to the seed*/


void Seed(unsigned long int s)
{
/*This chooses the generator to be used ("taus")
This is a maximally equidistributed combined Tausworthe generator by L'Ecuyer. The sequence is,

       x_n = (s1_n ^^ s2_n ^^ s3_n)

       where,

       s1_{n+1} = (((s1_n&4294967294)<<12)^^(((s1_n<<13)^^s1_n)>>19))
       s2_{n+1} = (((s2_n&4294967288)<< 4)^^(((s2_n<< 2)^^s2_n)>>25))
       s3_{n+1} = (((s3_n&4294967280)<<17)^^(((s3_n<< 3)^^s3_n)>>11))

       computed modulo 2^32. In the formulas above ^^ denotes "exclusive-or".
       Note that the algorithm relies on the properties of 32-bit unsigned integers and has
       been implemented using a bitmask of 0xFFFFFFFF to make it work on 64 bit machines.

       The period of this generator is 2^88 (about 10^26). It uses 3 words of state per generator.
       For more information see,
              P. L'Ecuyer, "Maximally Equidistributed Combined Tausworthe Generators",
              Mathematics of Computation, 65, 213 (1996), 203--213.*/

/*Allocates space for a generator and sets the seed*/


	r = gsl_rng_alloc(GENERATOR);

        if (s == 0) /*to use a seed taken from the calendar*/
        		sd = (unsigned long int) time(NULL);
	else
        		sd = s;

        gsl_rng_set(r, sd);
}

unsigned long int rt_seed()
{
	return sd;
}



/*funcion que regrea sa el factorial de el numero dado*/
double Fac(int t)
{
	double x = 1.0;

	for(int u = t; u > 0; u--)
		x *= (double) u;
	return x;
}

/*funcion que realiza la suma de los logaritmos de un numero dado log(n) mas
log(n-1) asi susecivamente has llegar a log(n-x+1), esto es log(n!/(n-x)!)*/
double Ifac(int n, int x)
{
	double S = 0.0;
	
	for(int i=n; i>n-x; i--)
		S += log((double) i);
	return S;
}

/*funcion que calcula el valor de los coeficientes del bimonio de Newton*/
double Combins(int n, int x)
{
	int i, pro=1, fac =1;

        if (x == n) 
        	return 1.0;
        if (x == n-1) 
        	return (double) n;

	for( i=1; i<=x; i++)
	{
		pro *= (n-i+1);
		fac *= i;
	}

	return (double) (pro/fac);  /*Division entera*/
}



double Un01(void)  /*Un01() */
{
	return gsl_rng_uniform(r);
}



double Nordens(double m, double std, double x)
{
	return gsl_ran_gaussian_pdf(x-m, std);
}

double Norsim(double m, double std)
{
	return (m + gsl_ran_gaussian(r, std));
}



/* Regresa el valor de ln(Gamma(x)) para x>0.*/

double gammlm(double x)
{
        return gsl_sf_lngamma(x);

}


/*Funcion Gama*/
double Gamma(double x)
{
        return gsl_sf_gamma(x);

}

double Gammadens(double a, double b, double x) /*Gamma: K x^{a-1} e^{-bx}*/
{
	return gsl_ran_gamma_pdf( x, a, 1.0/b);
}


/*Simulacion de la Gamma.*/
double Gammasim(double a, double b)  /*Gamma: K x^{a-1} e^{-bx}*/
{
	return gsl_ran_gamma(r, a, 1.0/b);
}


/*se calcula el valor de la funcion Beta con los parametros dados*/
double Beta(double v, double w)
{
	return (exp(gammlm(v) + gammlm(w) - gammlm(v+w)));
}



/*Densidad Beta*/
double Betadens(double v, double w, double q)
{
	return gsl_ran_beta_pdf( q, v, w);
}




/*Se convierten a y b a enteros...solo es bueno para parametros mayores o
iguales a 1*/
double Betasim(double a, double b)
{
	return gsl_ran_beta( r, a, b);
}






/*Funcion de probabilidad binomial*/
double BnmPrb(double p, unsigned int n, unsigned int x)
{
	return gsl_ran_binomial_pdf( x, p, n);
}


/*Simulacion de una binomial*/
int Bnmsim(double p, unsigned int n)
{
	return gsl_ran_binomial( r, p, n);
}



